import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

#---------------------------------------------------------------------------------------
# Este material é apenas introdutório, está imcompleto e precisa ser complementado para
# ficar nivelado com o script disponível em R!
# Elaborado por Vivian Kesse (aluna de Ciência da Computação/UESC) em 2024.2
#---------------------------------------------------------------------------------------

# Dados das turmas
turma_A = np.array([4, 5, 5, 6, 6, 7, 7, 8])
turma_B = np.array([1, 2, 4, 6, 6, 9, 10, 10])
turma_C = np.array([0, 6, 6, 7, 7, 7, 7.5, 7.5])

# Funções para cada medida de dispersão


def amplitude_total(x):
    return np.max(x) - np.min(x)


def amplitude_interquartil(x):
    return np.percentile(x, 75) - np.percentile(x, 25)


def desvio_medio(x):
    return np.mean(np.abs(x - np.mean(x)))


def desvio_quadratico_medio(x):
    return np.mean((x - np.mean(x)) ** 2)

# ddof é p ajuste (grau de liberdade)


def variancia(x, amostral=True):
    ddof = 1 if amostral else 0
    return np.var(x, ddof=ddof)


def desvio_padrao(x, amostral=True):
    return np.sqrt(variancia(x, amostral))


def coeficiente_variacao(x):
    return (desvio_padrao(x) / np.mean(x)) * 100

# Função para calcular todas as medidas e retornar um dicionário com resultados e descrições


def calcular_medidas(turma, nome_turma):
    medidas = {
        'Amplitude Total': (amplitude_total(turma)),
        'Amplitude Interquartil': (amplitude_interquartil(turma)),
        'Desvio Médio': (desvio_medio(turma)),
        'Desvio Quadrático Médio': (desvio_quadratico_medio(turma)),
        'Variância Amostral': (variancia(turma, amostral=True)),
        'Variância Populacional': (variancia(turma, amostral=False)),
        'Desvio Padrão Amostral': (desvio_padrao(turma, amostral=True)),
        'Desvio Padrão Populacional': (desvio_padrao(turma, amostral=False)),
        'Coeficiente de Variação': (coeficiente_variacao(turma)),
    }
    print(f"\n--- {nome_turma} ---")
    for conceito, (valor) in medidas.items():
        print(f"{conceito}: {valor:.2f}")
    return medidas


# Calcular e exibir os resultados para cada turma
medidas_A = calcular_medidas(turma_A, 'Turma A')
medidas_B = calcular_medidas(turma_B, 'Turma B')
medidas_C = calcular_medidas(turma_C, 'Turma C')

# Visualizações para a apresentação

# 1. Preparação dos dados para gráficos
dados = pd.DataFrame({
    'Turma': ['A']*len(turma_A) + ['B']*len(turma_B) + ['C']*len(turma_C),
    'Notas': np.concatenate([turma_A, turma_B, turma_C])
})

# 2. Histograma da Distribuição de Notas
plt.figure(figsize=(8, 6))
sns.histplot(data=dados, x='Notas', hue='Turma', multiple='dodge', bins=10)
plt.title("Distribuição das Notas por Turma")
plt.xlabel("Notas")
plt.ylabel("Frequência")
plt.show()

# 3. Boxplot para visualizar a dispersão e amplitude interquartil
plt.figure(figsize=(8, 6))
sns.boxplot(x='Turma', y='Notas', data=dados)
plt.title("Boxplot das Notas por Turma")
plt.xlabel("Turma")
plt.ylabel("Notas")
plt.show()

# 4. Diagrama de barras para comparar variância e desvio padrão
variancia_desvio = pd.DataFrame({
    'Turma': ['A', 'B', 'C'],
    'Variância Amostral': [medidas_A['Variância Amostral'], medidas_B['Variância Amostral'], medidas_C['Variância Amostral']],
    'Desvio Padrão Amostral': [medidas_A['Desvio Padrão Amostral'], medidas_B['Desvio Padrão Amostral'], medidas_C['Desvio Padrão Amostral']]
})

variancia_desvio_melt = variancia_desvio.melt(id_vars="Turma", value_vars=["Variância Amostral", "Desvio Padrão Amostral"],
                                              var_name="Medida", value_name="Valor")

plt.figure(figsize=(8, 6))
sns.barplot(x='Turma', y='Valor', hue='Medida',
            data=variancia_desvio_melt, dodge=True)
plt.title("Comparação da Variância e Desvio Padrão por Turma")
plt.xlabel("Turma")
plt.ylabel("Valor")
plt.show()
