# Alunos: Ariel Narciso e Breno Moreira
# Professor: Jos Cludio Faria
# Medidas de Disperso
# 2019.2
# Atualizado: 2024-11-18 11:49:36 (Prof. J.C.Faria)

#---------------------------------------------------------------------------------
#.. Funo           Medida                             Pacote/implementao
#---------------------------------------------------------------------------------
#   at               Amplitude total                    vetor: implementada
#                                                         fdt: implementada
#   aiq              Amplitude interquartl (e semi)    vetor: implementada
#                                                         fdt: implementada
#   dm               Desvio mdio                       vetor: implementada
#   dqm              Desvio quadrtico mdio            vetor: implementada
#   var              Varincia                          vetor*: stats
#                                                         fdt: fdth
#   sd               Desvio padro                      vetor*: stats
#                                                         fdt: fdth
#   dpr              Desvio padro relativo             vetor: implementada
#   cv               Coeficiente de variao            vetor: implementada
#                                                         fdt: implementada
#   em               Erro padro da mdia               vetor: implementada
#---------------------------------------------------------------------------------
# * Pode ser usado para data.frames e matrizes tambm

#.. Carregando os pacotes necessrios
#install.packages('fdth')
library(fdth)

#. Vetores
#.. Implementao de funes

#... Amplitude total
at <- function(x)
{
  print(diff(range(x)))
  print(range(x))
}

#... Amplitude interquartil
# argumento: siq = semi-interquartil
aiq <- function(x,
                siq=FALSE)
{
  res <- quantile(x)[4] - quantile(x)[2]

  ifelse(siq,
         res/2,
         res)
}

#... Desvio mdio
dm <- function(x)
{
  sum(abs(x-mean(x))) / length(x)
}

#... Desvio quadrtico mdio
dqm <- function(x)
{
  sum((x-mean(x))^2) / length(x)
}

#... Desvio padro relativo
dpr <- function(x)
{
  sd(x) / mean(x)
}

#... Coeficiente de variao
cv <- function(x)
{
  100 * sd(x) / mean(x)
}

#... Erro padro da mdia
em <- function(x)
{
  print(sd(x) / sqrt(length(x)))
}

#.. Reproduzindo amostras apresentadas no slide
(A <- c(2, 1.2, 2.1, 1.6, 0.9, 2.2, 1.8))    # Altura, m
(B <- c(1.8, 1.7, 1.7, 1.7, 1.5, 1.7, 1.5))  # Altura, m

#.. Visualizao grfica das amostras
par(mfrow=c(1, 2),
    bty='l')

# A
plot(A,
     type='h',
     ylim=c(0, 2.5),
     xlab='Amostra A',
     ylab='Altura (m)',
     col = 'darkblue',
     lwd=2)

# Traando a mdia de A
abline(h=mean(A),
       col='red',
       lwd=3)

# B
plot(B,
     type='h',
     ylim=c(0, 2.5),
     xlab='Amostra B',
     ylab='Altura (m)',
     col = 'darkgreen',
     lwd=2)

# Traando a mdia de B
abline(h=mean(B),
       col='red',
       lwd=3)

# Amplitude total
at(A)
at(B)

# Amplitude interquartlica
aiq(A)
aiq(B)

# Na unha
(quantile(A)[4] - quantile(A)[2])
(quantile(B)[4] - quantile(B)[2])


# Amplitude semi-interquartlica
aiq(A,
    siq=TRUE)
aiq(B,
    siq=TRUE)

# Na unha
(quantile(A)[4] - quantile(A)[2]) / 2
(quantile(B)[4] - quantile(B)[2]) / 2

# Desvio mdio
dm(A)
dm(B)

# Desvio quadrtico mdio
dqm(A)
dqm(B)

# Varincia
# Soma do quadrado dos desvios (SQD) calculada
# sem a necessidade de conhecer a mdia, ou seja, usando
# apenas os dados:
sum(A^2) - (sum(A)^2 / length(A))
sum(B^2) - (sum(B)^2 / length(B))

# Varincia na unha
(sum(A^2) - (sum(A)^2 / length(A))) / (length(A) - 1)
(sum(B^2) - (sum(B)^2 / length(B))) / (length(B) - 1)

var(A)
var(B)

# Desvio padro
sd(A)
sd(B)

# Desvio padro relativo
dpr(A)
dpr(B)

# Coeficiente de variao
cv(A)
cv(B)

# Erro padro da mdia
em(A)
em(B)

#. Tabela de distribuio de frequncias
#.. Implementao de funes

#... Amplitude total (tabela)
at_t <- function(x)
{
  # Verificao da classe de x: segurana apenas!
  if (!inherits(x,
                'fdt'))
    stop("O argumento x no  da classe 'fdt'.\n\n")

  pm_max <- x$breaks[['end']] -
           (x$breaks[['h']] / 2)

  pm_min <- x$breaks[['start']] +
           (x$breaks[['h']] / 2)

  pm_max - pm_min
}

#... Amplitude interquartil (tabela)
# argumento: siq = semi-interquartil
aiq_t <- function(x,
                  siq=FALSE)
{
  # Verificao da classe de x: segurana apenas!
  if (!inherits(x,
                'fdt'))
    stop("O argumento x no  da classe 'fdt'.\n\n")

  res <- (quantile(x,
            i=3) -
          quantile(x,
                   i=1))
  ifelse(siq,
         res/2,
         res)
}

#.. Fazendo objeto da classe fdt (pacote fdth)
(tb <- make.fdt(f=c(9, 9, 21, 32, 12, 6, 3, 3, 3, 2),
                start=3600,
                end=15600))

#.. Visualizao grfica das fdt
par(mfrow=c(1, 1),
    bty='l')

plot(tb,
     type='fh',
     xlab = 'Altura (m)',
     ylab = 'Frequncia');

# Amplitude total (tabela)
at_t(tb)

# Amplitude interquartil (tabela)
aiq_t(tb)

# Amplitude semi-interquartil (tabela)
aiq_t(tb,
      siq=TRUE)

# Varincia (tabela)
var(tb)

# Desvio padro (tabela)
sd(tb)

# Coeficiente de variao (tabela)
100 * sd(tb)/mean(tb)
